# -*- coding: utf-8 -*-

import logging
import json

from odoo import api, models

_logger = logging.getLogger(__name__)

class VoipConfigurator(models.Model):
    _inherit = 'res.users'

    def _init_store_data(self, store):
        """
        Overrides the PBX config in order to support multi-providers, and introduce the
        notion of "voip.options". Example of configuration:

        - ir.config_parameters:
            voip.ip_pbx -> {
                "axivox": "odoo.axivox.com",
                "allocloud": "odoo.allocloud.com",
                "default": "odoo.axivox.com"
            }
            voip.wsServer -> {
                "axivox": "wss://pabx.axivox.com:3443",
                "allocloud": "wss://pabx.allocloud.com:3443",
                "default": "wss://pabx.axivox.com:3443"
            }
            voip.options -> {
                "other": {"hackWssInTransport": true,
                          },
                "default": {},
            }

        - sip_login of users
            User1 -> myLogin1   (will use default provider)
            User2 -> myLogin2@Allocloud

        """
        super()._init_store_data(store)

        voipConfig = store.get_result().get('Store', {}).get('voipConfig', {})
        login = self.env.user._is_internal() and self.env.user.voip_username
        if not voipConfig or not login:
            return

        try:
            pbx_ip = json.loads(voipConfig.get("pbxAddress"))
            ws_server = json.loads(voipConfig.get("webSocketUrl"))

            get_param = self.env["ir.config_parameter"].sudo().get_param
            sip_options = json.loads(get_param("voip.options", "{}"))
        except (AssertionError, json.decoder.JSONDecodeError):
            _logger.exception("Failed to decode voip config settings")
            return

        provider = login.split('@')[1] if '@' in login else None
        voipConfig.update({
            "pbxAddress": pbx_ip.get(provider) or pbx_ip.get("default"),
            "webSocketUrl": ws_server.get(provider) or ws_server.get("default"),
            "sipOptions": sip_options.get(provider) or sip_options.get("default"),
        })
        store.add({"Store": {"voipConfig": voipConfig}})
